#include ".\ball.h"


Ball::Ball(std::string filename, Physics* physics):GameObject(filename.c_str(), physics)
{
	dBodySetGravityMode (m_body, 1);
	m_radius = 0.008f;
	memset (m_geom,0,sizeof(m_geom));

//	setPosition(1, 1, 10);	// initial position (where should we put it?)

//	setPosition(0.1f, 0.1f, 0.1f);
//	setPosition(0.0f, (m_length/2)*0.7, 0.05f);
	setPosition(0.01f, 0.05f, 0.02f);

	// x nach rechts
	// y nach oben

	dMass m;	

	//Create sphere	
	dMassSetSphere (&m,3.0f,m_radius);		// Set the mass parameters to represent a sphere of the given radius and density
	m_geom[0] = m_physics->createSphere(m_radius);
	m_physics->setBallGID(m_geom[0]);

    for (int k=0; k < GPB; k++) 
	{
      if (m_geom[k]) 
		  dGeomSetBody (m_geom[k],m_body);	// sets the body associated with a placeable geom
    }
    
    dBodySetMass (m_body,&m);	// Sets the mass of the body

	// Init the GL stuff
    int i, slices = 32;
    int j, stacks = 16;

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

    m_gllist = glGenLists(1);
    
    glNewList(m_gllist, GL_COMPILE);
    {
        for (i = 0; i < stacks; i++)
        {
            float k0 = (float)  i      / stacks;
            float k1 = (float) (i + 1) / stacks;
 
            float s0 = sinf(pi * (k0 - 0.5));
            float c0 = cosf(pi * (k0 - 0.5));
            float s1 = sinf(pi * (k1 - 0.5));
            float c1 = cosf(pi * (k1 - 0.5));

            glBegin(GL_QUAD_STRIP);
            {
                for (j = 0; j <= slices; j++)
                {
                    float k = (float) j / slices;
                    float s = sinf(pi * k * 2.0);
                    float c = cosf(pi * k * 2.0);

                    glTexCoord2f(k, k0);
                    glNormal3f(s * c0, c * c0, s0);
                    glVertex3f(s * c0, c * c0, s0);

                    glTexCoord2f(k, k1);
                    glNormal3f(s * c1, c * c1, s1);
                    glVertex3f(s * c1, c * c1, s1);
                }
            }
            glEnd();
        }
    }
    glEndList();
}

Ball::~Ball(void)
{
    if (glIsList(m_gllist))
        glDeleteLists(m_gllist, 1);
}

bool Ball::hasValidPosition(){
	if(m_position[1]<-0.09)
		return false;
	return true;
}


void Ball::addForceXYZ(double x, double y, double z)
{
	dBodyAddForce(m_body, x, y, z);	
}


void Ball::stop()
{
	dBodySetLinearVel(m_body, 0.0f, 0.0f, 0.0f);

	dBodySetAngularVel(m_body, 0.0f, 0.0f, 0.0f);
}

void Ball::draw()
{

	glPushMatrix();
//	glTranslatef(m_position[0], m_position[1], m_position[2]);
	setTransform(m_position,m_rotation);


//	glScalef(0.02f, 0.02f, 0.02f);
	glScalef(m_radius, m_radius, m_radius);
    glPushAttrib(GL_POLYGON_BIT |
                 GL_LIGHTING_BIT |
                 GL_DEPTH_BUFFER_BIT);
    {
        static const float  s[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
        static const float  e[4] = { 0.2f, 0.2f, 0.2f, 1.0f };
        static const float  h[1] = { 64.0f };

        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,  s);
        glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,  e);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, h);

        glEnable(GL_COLOR_MATERIAL);

        glBindTexture(GL_TEXTURE_2D, m_texture);

        /* Render the ball back to front in case it is translucent. */

        glDepthMask(GL_FALSE);

        glCullFace(GL_FRONT);
        glCallList(m_gllist);
        glCullFace(GL_BACK);
        glCallList(m_gllist);

        /* Render the ball into the depth buffer. */

        glDepthMask(GL_TRUE);
        glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
        {
            glCallList(m_gllist);
        }
        glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);

        /* Ensure the ball is visible even when obscured by geometry. */

        glDisable(GL_DEPTH_TEST);

        glColor4f(1.0f, 1.0f, 1.0f, 0.3f);
        glCallList(m_gllist);
    }
    glPopAttrib();
	glPopMatrix();
}
