#include ".\gamecontroller.h"
#include <SDL.h>


//#include <iostream>
using namespace std;

GameController::GameController(void)
{
	m_VideoFlags = 0;
	m_MainWindow = NULL;

	// initialize tracker: 1) Camera
	ImageSource * source = NULL;
	// aquire camera
	source = new BCamera();
//	source = new BPicture("E:\\prog-zone\\my_sources\\arPraktikum\\svn\\buidl\\buidl1.jpg");

	// 2) tracker
	MarkerTracker * tracker = NULL;
	tracker = new MarkerTracker();
	tracker->setSource(source);

	tracker->readMarkerConfig("marker.cfg");

	m_input = new Input();
	// set flipper-codes and tracker
	m_input->setTracker(tracker, 0xB3CE, 0x8DFF);

	m_physics = new Physics();

	m_viz = new Visualization(SCREEN_WIDTH, SCREEN_HEIGHT, tracker, source, m_physics);
	m_sound = new Sound();

	m_physics->setSound(m_sound);

	m_balls_left = 2;
	m_viz->setBallsLeft(m_balls_left);
}

GameController::~GameController(void)
{
	delete m_input;
	delete m_viz;
	delete m_sound;
	delete m_physics;
}

void GameController::mainloop(void)
{
	bool done = false;                                     

	while(! done)                                          
	{
		//if (m_input->ProgressEvent() == 1)
		//	done = true;

		SDL_Event event;

		while( SDL_PollEvent(& event) )                    
		{
			switch ( event.type )                          
			{
			case SDL_QUIT :                                        
				done = true;                                        
				break;

			case SDL_KEYDOWN :                                      
				switch(event.key.keysym.sym)
				{
				case SDLK_ESCAPE:
					done = true;
					break;
				}
				break;

			case SDL_KEYUP :                                                  
				switch(event.key.keysym.sym)
				{
				case 'm':
					if (m_viz->GetCurrentMode() == 0)
					{
						m_balls_left = 2;
						m_viz->setBallsLeft(m_balls_left);
						m_viz->m_ball->stop();
						m_viz->m_ball->setPosition(0.01*dRandReal(), 0.05f, 0.02f);
						m_viz->SetModeGame();
						m_sound->fadeInMusic(1000);
						SDL_WM_SetCaption("Game", "Game");
					}else
					{
						m_viz->SetModeGUI();
						m_sound->fadeOffMusic(1000);
						SDL_WM_SetCaption("GUI", "GUI");
					}
					//	m_sound->playShot();
					break;
				case 'q':
					m_viz->m_tracker->incThreshold();
					break;
				case 'w':
					m_viz->m_tracker->decThreshold();
					break;

					// stefan: im not yet sure which function we should use
				case 'u':	// left panel up
				//	m_viz->m_ball->addForceXYZ(0.0,0.1, 0);
				//	m_viz->getLeftPanel()->addTorqueXYZ(0.0,1,0);
					m_viz->getLeftPanel()->addForceXYZ(0.0,0.2,0);
					m_sound->playLeftPad();
				//	m_viz->getLeftPanel()->addTorqueOnAxis(-0.01);
				
					break;
				case 'i':	// right panel up
				//	m_viz->m_ball->addForceXYZ(-1.0, 2.0, 0);
				//	m_viz->getRightPanel()->addTorqueXYZ(0.0,1,0);
					m_viz->getRightPanel()->addForceXYZ(0.0,0.2,0);
					m_sound->playRightPad();
				//	m_viz->getRightPanel()->addTorqueOnAxis(0.01);
					break;

				case 'r':
					// reset ball position
					m_viz->m_ball->setPosition(0.01*dRandReal(), 0.05f, 0.02f);
					m_balls_left--;
					m_viz->setBallsLeft(m_balls_left);
					break;
				}
				break;

			case SDL_VIDEORESIZE :                                                 
				m_viz->SizeOpenGLScreen(event.resize.w, event.resize.h);   
				break;

			default:                                    
				break;                                  
			} 
		} 
		
		// calculate new pose data
		m_viz->m_tracker->update();

		// only do something if at least one marker is tracked
		if (m_viz->m_tracker->markerIsTracked(MARKER_BOARD))
		{
			// update input from tracker
			m_input->update();
			if (m_viz->GetCurrentMode() == GAME_PLAYING)
			{
				if (m_input->leftFlipperTriggered())
				{
					// update flipper
					// update physics
					cout << " L ";
					m_viz->getLeftPanel()->addForceXYZ(0.0,0.1,0);
					m_sound->playLeftPad();

				}else{ // if (m_input->leftFlipperTriggered())
					m_viz->getLeftPanel()->setZRotation(-0.7f);
					m_sound->setLeftPadIsDown();
				}

				if (m_input->rightFlipperTriggered())
				{
					// update flipper
					// update physics
					cout << " R ";
					m_viz->getRightPanel()->addForceXYZ(0.0,0.1,0);
					m_sound->playRightPad();
				}else{
					m_viz->getRightPanel()->setZRotation(0.7f);
					m_sound->setRightPadIsDown();
				}

				if(!m_viz->m_ball->hasValidPosition())
				{
						m_viz->m_ball->setPosition(0.01*dRandReal(), 0.05f, 0.02f);
						m_balls_left--;
						m_viz->setBallsLeft(m_balls_left);
				}
			}
			// simulate step
			m_physics->simLoop(m_viz->GetCurrentMode());


			m_viz->updateObjects();
		} // if markers are tracked
		m_viz->display();
	} 
}

void GameController::CreateMyWindow(const char * strWindowName, int width, int height)
{
    m_MainWindow = SDL_SetVideoMode(width, height, SCREEN_DEPTH, m_VideoFlags);     

    if( m_MainWindow == NULL )                            
    {
		std::cerr << "Failed to Create Window : " << SDL_GetError() << std::endl;         
        SDL_Quit();
    }
    SDL_WM_SetCaption(strWindowName, strWindowName);  


}

void GameController::SetupPixelFormat(void)
{
    m_VideoFlags    = SDL_OPENGL;                                         
    m_VideoFlags   |= SDL_HWPALETTE;                      
    m_VideoFlags   |= SDL_RESIZABLE; 
	
	if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDYES)
	{
		m_VideoFlags   |= SDL_FULLSCREEN;
	}


    const SDL_VideoInfo * VideoInfo = SDL_GetVideoInfo();    

    if(VideoInfo == NULL)                                     
    {
        std::cerr << "Failed getting Video Info : " << SDL_GetError() << std::endl;          
        SDL_Quit();
    }

    if(VideoInfo -> hw_available)                      
        m_VideoFlags |= SDL_HWSURFACE;
    else
        m_VideoFlags |= SDL_SWSURFACE;

    
    if(VideoInfo -> blit_hw)                 
        m_VideoFlags |= SDL_HWACCEL;

    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );     
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE,   SCREEN_DEPTH);        
    SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0);   
	//SDL_GL_SetAttribute( SDL_GL_ALPHA_SIZE, 8);
    SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0);     
    SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0);   
    SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0);
    SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0);
}

void GameController::Init_Window()
{
    m_viz->init_gl();
	m_viz->init_objects();
    
    if ( SDL_EnableKeyRepeat(1, SDL_DEFAULT_REPEAT_INTERVAL) )	 
    {
        std::cout << "Failed enabling key repeat :" << SDL_GetError() << std::endl;
        SDL_Quit();
    }  
    
}

