#include ".\gameobject.h"


#include <iostream>
using namespace std;

GameObject::GameObject(Physics* physics)
{
	m_physics = physics;	
	//m_body = Physics::Instance().createBody(); 
	m_body = m_physics->createBody();
	
	dBodySetGravityMode (m_body, 0);	// gravity does not influence body
	
	m_Filename = "";
	m_texture = 0;
	pi = 3.14159265359;
	setBoardSize();
}

GameObject::GameObject(const std::string & Filename, Physics* physics)
{
	m_Filename = Filename;
	m_texture = 0;
    cout << "Loading " << Filename << " to textureID "<< LoadGLTextures();
	
	m_physics = physics;
	m_body = m_physics->createBody();
	dBodySetGravityMode (m_body, 0);	// gravity does not influence body
	pi = 3.14159265359;
	setBoardSize();
}

void GameObject::coutPos(){
//	const dReal* position	= dGeomGetPosition(m_geom[0]);  
//	std::cout << "x: " << position[0] << "y: " << position[1] << "z: " << position[0] << std::endl;
	for(int i=0; i<80; i++){
		printf("\b");
	}
	printf ("x: %f y: %f z: %f", m_position[0], m_position[1], m_position[2]);
}
void GameObject::setBoardSize()
{
	m_length=0.2f;
	m_width=0.2f;
	m_height=0.04f;
}

GameObject::~GameObject(void)
{
	glDeleteTextures(1, &m_texture);
}

void GameObject::setZRotation(float angle)
{
  dMatrix3 R;
  dRFromAxisAndAngle (R, 0, 0, 1, angle);
  dGeomSetRotation (m_geom[0], R);
}

void GameObject::setRotation(const double* R)
{
  dGeomSetRotation (m_geom[0], R);
}



int GameObject::LoadGLTextures()
{
	GLenum before2 = glGetError();
	
	int Status=FALSE;									
	//Load Bitmap
	SDL_Surface* bmpFile = SDL_LoadBMP(m_Filename.c_str());
	
	if (bmpFile != NULL)
	{
		if (bmpFile->w < 1) 
		{
			SDL_FreeSurface(bmpFile);
			return false;
		} 
		else if ( !((bmpFile->w & (bmpFile->w-1)) == 0) )
		{
			SDL_FreeSurface(bmpFile);
			return false;
		}    

		glPixelStorei(GL_UNPACK_ALIGNMENT,4);

		m_texture = 0;
		glGenTextures(1,&m_texture);

		if (m_texture == 0)
		{
			throw "Could not load texture...";
		}
		glBindTexture(GL_TEXTURE_2D,m_texture);

		// TEST
		//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

		//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		
		
		//gluBuild2DMipmaps(GL_TEXTURE_2D,3,bmpFile->w,
		//	bmpFile->h,GL_BGR_EXT,
		//	GL_UNSIGNED_BYTE,bmpFile->pixels);

		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, bmpFile->w, bmpFile->h, 0, GL_BGR_EXT, GL_UNSIGNED_BYTE, bmpFile->pixels);

		//Free surface after using it
		SDL_FreeSurface(bmpFile);
		Status = true;
	} 
	else
	{
		Status = false;
	}
	return Status;										// Return The Status
}

GLuint GameObject::GetTexture()
{
	return m_texture;
}

const double* GameObject::getPosition()
{ 
	return dBodyGetPosition (m_body); 
}

const double* GameObject::getRotation()
{ 
	return dBodyGetRotation (m_body); 
}

void GameObject::stopZRotating()
{
	dBodySetAngularVel(m_body, 0.0f, 0.0f, 0.0f);
}

void GameObject::setPosition(double x, double y, double z)
{
	m_position[0] = (float)x;
	m_position[1] = (float)y;
	m_position[2] = (float)z;
	dBodySetPosition(m_body, x,y,z);			
}


void GameObject::update()
{
	
	const double *pos = getPosition();
	const double *rot = getRotation();
	
	for(int i=0; i<2; i++)
	{
		m_position[i] = (float)pos[i];
	}
	m_position[2] = 0.0f;	// set z-coordinate to zero
	dBodySetPosition(m_body, m_position[0],m_position[1],0.0);			

	for(int i=0; i<12; i++){
		m_rotation[i] = (float)rot[i];
	}

}
	
dBodyID GameObject::getBodyID()
{ 
	return m_body;
}

void GameObject::draw()
{
	// do nothing here, will be overwritten in sub-classes
}	



void GameObject::setTransform (const float pos[3], const float R[12])
{
  GLfloat matrix[16];
  matrix[0]=R[0];
  matrix[1]=R[4];
  matrix[2]=R[8];
  matrix[3]=0;
  matrix[4]=R[1];
  matrix[5]=R[5];
  matrix[6]=R[9];
  matrix[7]=0;
  matrix[8]=R[2];
  matrix[9]=R[6];
  matrix[10]=R[10];
  matrix[11]=0;
  matrix[12]=pos[0];
  matrix[13]=pos[1];
  matrix[14]=pos[2];
  matrix[15]=1;
  glMultMatrixf (matrix);
}

void GameObject::drawCylinder(float radius, float length){
//	glutSolidCylinder(radius, length, 15, 15);
GLUquadricObj* cylinder = gluNewQuadric();

int numDivAroundZ=15, numDivAlongZ=5;

gluCylinder(cylinder,radius,radius,length,numDivAroundZ,numDivAlongZ); 

}


void GameObject::drawBox (const float sides[3])
{
  float lx = sides[0]*0.5f;
  float ly = sides[1]*0.5f;
  float lz = sides[2]*0.5f;

  // sides
  glBegin (GL_TRIANGLE_STRIP);
//glNormal3f (-1,0,0);
  glVertex3f (-lx,-ly,-lz);
  glVertex3f (-lx,-ly,lz);
  glVertex3f (-lx,ly,-lz);
  glVertex3f (-lx,ly,lz);
//glNormal3f (0,1,0);
  glVertex3f (lx,ly,-lz);
  glVertex3f (lx,ly,lz);
//glNormal3f (1,0,0);
  glVertex3f (lx,-ly,-lz);
  glVertex3f (lx,-ly,lz);
//glNormal3f (0,-1,0);
  glVertex3f (-lx,-ly,-lz);
  glVertex3f (-lx,-ly,lz);
  glEnd();

  // top face
  glBegin (GL_TRIANGLE_FAN);
//glNormal3f (0,0,1);
  glVertex3f (-lx,-ly,lz);
  glVertex3f (lx,-ly,lz);
  glVertex3f (lx,ly,lz);
  glVertex3f (-lx,ly,lz);
  glEnd();

  // bottom face
  glBegin (GL_TRIANGLE_FAN);
//glNormal3f (0,0,-1);
  glVertex3f (-lx,-ly,-lz);
  glVertex3f (-lx,ly,-lz);
  glVertex3f (lx,ly,-lz);
  glVertex3f (lx,-ly,-lz);
  glEnd();
}

	
void GameObject::dsDrawBoxD (const double pos[3], const double R[12], const double sides[3])
{
 glPushMatrix();
  int i;
  float pos2[3],R2[12],fsides[3];
  for (i=0; i<3; i++) pos2[i]=(float)pos[i];
  for (i=0; i<12; i++) R2[i]=(float)R[i];
  for (i=0; i<3; i++) fsides[i]=(float)sides[i];
  
  setTransform (pos2,R2);
  drawBox (fsides);
  glPopMatrix(); 
}

	
void GameObject::dsDrawCyl (const double pos[3], const double R[12],
		 float radius, float length)
{
 glPushMatrix();
  int i;
  float pos2[3],R2[12];
  for (i=0; i<3; i++) pos2[i]=(float)pos[i];
  for (i=0; i<12; i++) R2[i]=(float)R[i];
  
  setTransform (pos2,R2);
  drawCylinder (radius, length);
  glPopMatrix(); 
}