#pragma once

#include "SDL.h"
#include <gl\glaux.h>		// Header File For The Glaux Library
#include <gl\gl.h>
#include <stdio.h>			// Header File For Standard Input/Output
#include <string>			// much! better than char*
#include ".\ode\ode.h"		// Header file for the Open Dynamics Engine
#include "physics.h"

#define GPB 6			// maximum number of geometries per body

class GameObject
{
protected:
	GLuint	m_texture;	// Currently only on texture per Object is supported
	std::string m_Filename;		// Currently only on texture per Object is supported
	
	float m_length;		// board length
	float m_width;		// board width
	float m_height;		// board height
	float m_radius;

protected:
	dBodyID m_body;				// the body
	dGeomID m_geom[GPB];		// geometries representing this body
	float	m_position[3];		// position in world cordinates
	float	m_rotation[12];		// current rotation matrix
	Physics *m_physics;			// Physics engine
	double pi;

private:
	void setBoardSize(void);

public:
	/** C'tor with filename
	* @param Filename Path to the picture (bmp atm)
	* @param physics Pointer to physics engine
	*/
	GameObject(const std::string & Filename, Physics* physics);
	/** C'tor w/o texture */
	GameObject(Physics* physics);
	GameObject();	///< do not call this constructor, because we need a physics engine...
	/** D'tor */
	virtual ~GameObject();
	/** Loads the texture
	* @return texture-ID (openGL)
	* @todo make private
	* @todo return-type should be bool!
	*/
	int LoadGLTextures();

	/** Return texture id
	* @return texture id
	*/
	GLuint GetTexture();

	virtual void update();
	virtual void draw();		// draws the object
	void setPosition(double x, double y, double z);
	void setRotation(const double* R);
	void setZRotation(float angle);

	const double* getPosition();	
	const double* getRotation();	
	dBodyID	getBodyID();
		void coutPos();

		void stopZRotating();

		// only for testing
		void setTransform (const float pos[3], const float R[12]);
	

		void drawBox (const float sides[3]);

		void drawCylinder(float radius, float length);
		void dsDrawBoxD(const double pos[3], const double R[12], const double sides[3]);
	
		void dsDrawCyl(const double pos[3], const double R[12],	 float radius, float length);

};


