#pragma once
#include ".\ode\ode.h"
#include "Sound.h"
//#include "singleton.h"

#define MAX_CONTACTS 4          // maximum number of contact points per body


// stefan: i would like to call this class "God", because it controls the laws of physic in our world
class Physics //: public Singleton<Physics>
{
public:
	Physics(void);
	~Physics(void);
	bool init();
	
	double*		getGravity();
	void		setGravity(double x, double y, double z);

	/*Wrapper ODE functions */
	dBodyID		createBody();
	dGeomID		createSphere(float radius);
	dGeomID		createBox(float length, float width, float height);
	dGeomID		createPlane(dReal a, dReal b, dReal c, dReal d); 
	dGeomID		createCCyl(float radius, float length);
	dGeomID		createCyl(float radius, float length);


	dJointID createFixedJoin(void);
	dJointID createHingeJoin(void);

	void simLoop(int run);

	// This wrapper is called by the physics library to get information 
	// about object collisions
	static void collisionCallback(void *pData, dGeomID o1, dGeomID o2);
	
	// This actually implements the collision callback 

	void collisionCallback (dGeomID o1, dGeomID o2);
	void setBallGID(dGeomID ball);
	void resetBumperHits();
	void increaseBumperHits();
	int getBumperHits();

	void setSound(Sound *sound);
private:
	dWorldID		m_world;
	dSpaceID		m_space;	
	dJointGroupID	m_contactgroup;
	double			m_gravity[3];
	
	int bumperHits;

	Sound *m_sound;

	dGeomID		m_ball;
	
};
