#include ".\visualization.h"
#include <string>

// Thorsten: Not really neccessary. C++ delete ist Nullpointer-safe. btw. free is not ;-)
// Basti: ok =) I use SAFE_DELETE as I use SAFE_RELEASE, which is by concept not null pointer safe =) good to know anyway.
#define SAFE_DELETE(p) if(p) { delete p; p = NULL; }

Visualization::Visualization(int width, int height, MarkerTracker * tracker, ImageSource * source, Physics* physics)
{
	// initialize all member variables
	m_width = width;
	m_height = height;
	m_mode = GAME_MENU;
	m_game_fading = 0;
	m_game_fade_speed = 0.1f;
	m_time_passed = 0;
	m_current_score = 0;
	m_current_bumperhits = 0;
	init_gl();

	m_tracker = tracker;
	m_source = source;
	m_physics = physics;

	m_GUI_background = NULL;	// stefan: is not really a game object, is it?
	m_board = NULL;
	m_panel_left = NULL;
	m_panel_right = NULL;
	m_bumper1 = m_bumper2 = m_bumper3 = NULL;
	m_leftBlocker = NULL;
	m_rightBlocker = NULL;
	m_gameover = NULL;
}

Visualization::~Visualization(void)
{
	SAFE_DELETE(m_GUI_background)
	SAFE_DELETE(m_board)
	SAFE_DELETE(m_panel_left);
	SAFE_DELETE(m_panel_right);
	SAFE_DELETE(m_bumper1);
	SAFE_DELETE(m_bumper2);
	SAFE_DELETE(m_bumper3);
	SAFE_DELETE(m_textscreen);
	SAFE_DELETE(m_leftBlocker);
	SAFE_DELETE(m_rightBlocker);
	SAFE_DELETE(m_gameover);
}

void Visualization::init_gl(void)
{
   if (m_mode == GAME_MENU)
   {
		glDisable(GL_LIGHTING);
		glDisable(GL_DEPTH_TEST);                             
		glDisable(GL_BLEND);
   } 
   else 
   {
		glEnable(GL_DEPTH_TEST);   
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
		
		// init lights 
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
		glEnable(GL_COLOR_MATERIAL);
   }

   glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
   glClearColor(0.0f, 0.0f, 0.0f, 0.0f); 
   glShadeModel(GL_SMOOTH); 
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
   SizeOpenGLScreen(m_width, m_height);                       
}

void Visualization::init_objects(void)
{
	// Here we init the objects
	m_GUI_background = new GameObject("../data/images/gui_bg2.bmp", m_physics);	
	m_board = new Board("../data/images/windows.bmp", m_physics);
	m_panel_left = new Panel("../data/images/panel.bmp", m_physics, -0.04f, -0.065f, 0.01f);
	m_panel_right = new Panel("../data/images/panel.bmp", m_physics, 0.04f, -0.065f, 0.01f);
	m_panel_left->setAngle(0);
	//m_panel_left->setPosition(-0.04, -0.085, 0.01);
	m_panel_right->setAngle(180);
//	m_panel_right->setPosition(0.04, -0.085, 0.01);

	// create bumpers
	m_bumper1 = new Bumper("../data/images/bumper.bmp", m_physics);
	m_bumper2 = new Bumper("../data/images/bumper.bmp", m_physics);
	m_bumper3 = new Bumper("../data/images/bumper.bmp", m_physics);
	// set initial positions
	m_bumper1->setPosition(-0.03f, 0.05f, 0.02f);
	m_bumper2->setPosition(0.03f, 0.03f, 0.02f);
	m_bumper3->setPosition(0.03f, 0.00f, 0.02f);

	m_ball = new Ball("../data/images/ball.bmp", m_physics);
	m_textscreen = new TextScreen("../data/images/goodluck.bmp");
	m_gameover = new TextScreen("../data/images/gameover.bmp");
	m_gameover->setAlpha(0.0f);	// invisible right now

	m_leftBlocker = new Box(m_physics, -0.08f, -0.065f, 0.01f);
	m_rightBlocker = new Box(m_physics, 0.08f, -0.065f, 0.01f);
}

void Visualization::RenderString(float x, float y, float z, const char *string)
{
  char *c;
  glRasterPos3f(x, y,z);
  for (c=(char*)string; *c != '\0'; c++) 
  {
    glutBitmapCharacter(GLUT_BITMAP_8_BY_13, *c);
  }
}

void Visualization::fadeToBlack(void)
{
	m_game_fading = -1.0f;
}
void Visualization::fadeToWhite(void)
{
	m_game_fading = 1.0f;
}


void Visualization::display(void)
{

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	
	switch(m_mode)
	{
	case GAME_MENU:
		displayGUI();
		break;
	case GAME_PLAYING:
		displayGame();
		displayHUD();

		if (m_game_fading != 0.0)
			fadeGame();
		break;
	}

	glFlush();	

	SDL_GL_SwapBuffers();	
}


void Visualization::displayHUD(void)
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
//	gluOrtho2D(0, 1, 0, 1);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// TR, TODO: better with glPushAttrib
    glDisable(GL_DEPTH_TEST);
	// BG, Disabling DEPTH_TEST only not updates the depth-buffer,
	// it has to be cleared for the effect you wanted.
	glClear(GL_DEPTH_BUFFER_BIT);

	glBegin(GL_POLYGON);
		glColor4f(0.0f, 0.0f, 1.0f, 0.5f);
	// QuickHack
		glVertex2f(-1.0f, 0.8f); //lower left
		glVertex2f(-1.0f, 1.0f); //upper left
		glColor4f(1.0f, 1.0f, 0.0f, 0.2f);
		glVertex2f(1.0f, 1.0f); //upper right
		glVertex2f(1.0f, 0.8f); //lower right
	glEnd();

    // Print how many Markers are tracked
	char temp[10] = {};
	itoa(m_tracker->markersTracked(), temp, 10);
	std::string str = "Tracked Markers: ";
	str += temp;
	glColor3f(1.0f, 1.0f, 0.0f);
	RenderString(-0.9f, 0.9f, 0.0f, str.c_str());

	if (m_balls_left >= 0)
	{
		m_current_score = (float) (timeGetTime() - m_time_passed)/1000;
	} else {
		if (m_gameover->getAlpha() == 0.0f)
		{
			m_gameover->setAlpha(0.8f);
		}
	}
	
    // Print how many time ist passed
	char temp1[30] = {};
	snprintf(temp1, sizeof(temp1), "%010.3f", m_current_score);
	std::string str1 = "Time passed: ";
	str1 += temp1;
	glColor3f(1.0f, 1.0f, 0.0f);
	RenderString(0.3f, 0.9f, 0.0f, str1.c_str());

    // Print how many balls are left
	char temp2[30] = {};
	if (m_balls_left >= 0)
	{
		itoa(m_balls_left, temp2, 10);
	} else {
		strcpy(temp2, "0");
	}
	std::string str2 = "Balls left: ";
	str2 += temp2;
	glColor3f(1.0f, 1.0f, 0.0f);
	RenderString(-0.3f, 0.9f, 0.0f, str2.c_str());

	// Print how often the bumpers were hit
	char temp3[20] = {};
	m_current_bumperhits = m_physics->getBumperHits();
	
	itoa(m_current_bumperhits, temp3, 10);

	std::string str3 = "Score: ";
	str3 += temp3;
	glColor3f(1.0f, 1.0f, 0.0f);
	RenderString(0.3f, 0.85f, 0.0f, str3.c_str());

	// Draw TextScreen
	if (m_textscreen->getAlpha() > 0.0f)
		m_textscreen->draw();
	
	if ((m_gameover->getAlpha() > 0.0f) && (m_balls_left < 0))
		m_gameover->draw();

	if (m_balls_left < 0)
	{
		if (m_gameover->getTimesSeen() > 2)
		{
			// Send an SDL Event to the Gamecontroller, so everything is handled from there
			SDL_Event event;
			event.type = SDL_KEYUP;
			event.key.keysym.sym = (SDLKey) 'm';
			SDL_PushEvent(&event);
		}
	}

	glEnable(GL_DEPTH_TEST);
	glPopMatrix();
}


void Visualization::fadeGame(void)
{
	if (m_game_fading > 0.0f)
	{
		m_currentAlpha = m_game_fading;
		m_game_fading -= m_game_fade_speed;
		if (m_game_fading < 0.0f)
			m_game_fading = 0.0f;
	} else if (m_game_fading < 0.0f) {
		m_currentAlpha= m_game_fading + 1.0f;
		m_game_fading += m_game_fade_speed;
		if (m_game_fading > 0.0f)
			m_game_fading = 0.0f;
	}

	// draw background-alpha-box
	glDisable(GL_DEPTH_TEST);
	drawAlphaBox();
	glEnable(GL_DEPTH_TEST);

}


Panel *Visualization::getLeftPanel()
{
	return 	m_panel_left;
}
Panel *Visualization::getRightPanel()
{
	return m_panel_right;
}


void Visualization::updateObjects(void)
{

		m_ball->update();
		if(!m_ball->hasValidPosition())
			printf("ball is out!");


		m_panel_left->setPosition(-0.04, -0.065, 0.01);	
		m_panel_right->setPosition(0.04, -0.065, 0.01);

		m_panel_left->update();
		m_panel_right->update();

		// set position of bumpers according to tracked markers
		TransQuat bumper;
		if (m_tracker->markerIsTracked(MARKER_BUMPER1)) // bumper 1
		{
			if (m_tracker->calcIntraMarkerTransform(MARKER_BOARD, MARKER_BUMPER1, bumper))
			{
				m_bumper1->setPosition(bumper.t.x, bumper.t.y, bumper.t.z);
			}
		}
		if (m_tracker->markerIsTracked(MARKER_BUMPER2)) // bumper 2
		{
			if (m_tracker->calcIntraMarkerTransform(MARKER_BOARD, MARKER_BUMPER2, bumper))
			{
				m_bumper2->setPosition(bumper.t.x, bumper.t.y, bumper.t.z);
			}
		}
		if (m_tracker->markerIsTracked(MARKER_BUMPER3)) // bumper 3
		{
			if (m_tracker->calcIntraMarkerTransform(MARKER_BOARD, MARKER_BUMPER3, bumper))
			{
				m_bumper3->setPosition(bumper.t.x, bumper.t.y, bumper.t.z);
			}
		}

		m_bumper1->update();
		m_bumper2->update();
		m_bumper3->update();

		m_leftBlocker->setPosition(-0.08f, -0.045f, 0.01f);
		m_rightBlocker->setPosition(0.08f, -0.045f, 0.01f);

		m_leftBlocker->setZRotation(0.9f);
		m_rightBlocker->setZRotation(-0.9f);

		m_leftBlocker->update();
		m_rightBlocker->update();
}


// Function to display the game entities
void Visualization::displayGame(void)
{

	// TR, TODO: better with glPushAttrib
	glDisable(GL_DEPTH_TEST);
	displayImage();
	glEnable(GL_DEPTH_TEST);

	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(m_projectionMatrix);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	// scale picture and following objects accordingly
	float widthCorrection = static_cast<float>(m_width) / static_cast<float>(m_source->getWidth());
	float heightCorrection = static_cast<float>(m_height) / static_cast<float>(m_source->getHeight());
	glScalef(widthCorrection, heightCorrection, 1.0f);

	//TransQuat currMarker;
	//float hmarkerSize = 0.0f;

	// disable drawing of markers atm. because it breaks transparency

	//for (int i = 0; i < m_tracker->markersTracked(); i++)
	//{
	//	glPushMatrix();
	//	m_tracker->getMarkerPoseNumber(i, currMarker, hmarkerSize);
	//	glMultMatrixq(currMarker);
	//	// cover marker
	//	glNormal3d(0.0, 0.0, 1.0);

	//	glColor4f(1.0f, 0.0f, 0.0f, 0.2f);
	//	glBegin(GL_QUADS);
	//		glVertex2f(-hmarkerSize, -hmarkerSize);
	//		glVertex2f(-hmarkerSize, hmarkerSize);
	//		glVertex2f(hmarkerSize, hmarkerSize);
	//		glVertex2f(hmarkerSize, -hmarkerSize);
	//	glEnd();
	//	glPopMatrix();
	//}

	// Board Marker
	if (m_tracker->markerIsTracked(0x0066))
	{
		// board is tracked -> go onto it.
		glPushMatrix();
		TransQuat boardMarker;
		m_tracker->getMarkerPose(0x0066, boardMarker);
		glMultMatrixq(boardMarker);
		
		// and draw board.
		m_board->draw();
		
		m_panel_left->draw();
		m_panel_right->draw();

		m_bumper1->draw();
		m_bumper2->draw();
		m_bumper3->draw();

		m_ball->draw();


		m_leftBlocker->draw();
		m_rightBlocker->draw();

		// test box
		//Box test(m_physics, 0.0f, 0.0f, 0.0f);
		//test.draw();

		glPopMatrix();
	}

}

void Visualization::displayGUI(void)
{
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, m_GUI_background->GetTexture());

  glBegin(GL_QUADS);
	glColor3f(1.0f, 1.0f, 1.0f);
	// background with texture
	glTexCoord2i(0, 0);
	glVertex2i(-1, 1); //lower left
	glTexCoord2i(0, 1);
	glVertex2i(-1, -1); //upper left
	glTexCoord2i(1, 1);
	glVertex2i(1, -1); //upper right
	glTexCoord2i(1, 0);
	glVertex2i(1, 1); //lower right
  glEnd();

  glDisable(GL_TEXTURE_2D);

  if (m_current_score != 0.0f)
  {
	char temp1[30] = {};
	snprintf(temp1, sizeof(temp1), "%010.3f", m_current_score);
	std::string str1 = "You survived ";
	str1 += temp1;
	str1 += " seconds";
	glColor3f(1.0f, 1.0f, 0.0f);
	RenderString(-0.45f, -0.7f, 0.0f, str1.c_str());

	char temp2[20] = {};
	itoa(m_current_bumperhits, temp2, 10);
	std::string str2 = "Your last score was: ";
	str2 += temp2;
	str2 += " bumper hits";
	glColor3f(1.0f, 1.0f, 0.0f);
	RenderString(-0.45f, -0.8f, 0.0f, str2.c_str());

  }
}



void Visualization::SizeOpenGLScreen(int width, int height)            
{
    if (height==0)                                        
    {
        height=1;                                        
    }
	m_width = width;
	m_height = height;

	// init projection matrix
	float a = 1000.0f/999.0f;
	float b = 10.0f/999.0f;
	m_projectionMatrix[0] = 400.0f *2.0f / static_cast<float>(m_width);
	m_projectionMatrix[5] = 400.0f * 2.0f/ static_cast<float>(m_height);
	m_projectionMatrix[10] = a;
	m_projectionMatrix[11] = -1.0f;
	m_projectionMatrix[14] = b;
	m_projectionMatrix[1] = m_projectionMatrix[2] = m_projectionMatrix[3] = m_projectionMatrix[4] = 0.0f;
	m_projectionMatrix[6] = m_projectionMatrix[7] = m_projectionMatrix[8] = m_projectionMatrix[9] = 0.0f;
	m_projectionMatrix[12] = m_projectionMatrix[13] = m_projectionMatrix[15] = 0.0f;

    glViewport(0,0,m_width,m_height);                       
                                                                                                                                                                                                                          
    glMatrixMode(GL_PROJECTION);                        
    glLoadIdentity();                                   
    
	switch (m_mode)
	{
	case GAME_MENU:
		gluOrtho2D(0, 1, 0, 1);
		break;
	case GAME_PLAYING:
		// @todo remove : this will be overwritten in displayGAME()
		gluPerspective(45.0f,(GLfloat)m_width/(GLfloat)m_height, 0.5f ,150.0f);
		break;
	}
}

void Visualization::SetModeGame()
{
	m_gameover->resetTimesSeen();
	m_mode = GAME_PLAYING;
	fadeToWhite();
	init_gl();
	m_time_passed = timeGetTime();
}

void Visualization::SetModeGUI()
{
	m_mode = GAME_MENU;
	init_gl();
}

int Visualization::GetCurrentMode()
{
	return m_mode;
}

void Visualization::glLoadMatrixq(TransQuat t)
{
	glLoadMatrixf((math3d::createMatrix(t)).t);
}

void Visualization::glMultMatrixq(TransQuat t)
{
	glMultMatrixf((math3d::createMatrix(t)).t);
}

void Visualization::displayImage()
{
	// initialize modelview matrix
	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();
	// and draw picture
	float xAdjusted = static_cast<float>(m_width) / static_cast<float>(m_source->getWidth());
	float yAdjusted = static_cast<float>(m_height) / static_cast<float>(m_source->getHeight());
	glPixelZoom(xAdjusted, yAdjusted);
	// we are in the normalized cube!
	glRasterPos2i(-1,-1);
	glDrawPixels(m_source->getWidth(), m_source->getHeight(), GL_BGR_EXT, GL_UNSIGNED_BYTE, m_source->getBGRImage());
	glPopMatrix();
	glMatrixMode( GL_PROJECTION );
	glPopMatrix();
}

void Visualization::drawAlphaBox()
{
	// Doing a Fade by simply drawing a alpha box 
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	glBegin(GL_POLYGON);
	glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_TRUE);
	glColor4f(0.0f, 0.0f, 0.0f, m_currentAlpha);
	// QuickHack background
	glVertex2i(-1, -1); //lower left
	glVertex2i(-1, 1); //upper left
	glVertex2i(1, 1); //upper right
	glVertex2i(1, -1); //lower right
	glEnd();
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);

	glPopMatrix(); // pop modelview

	glMatrixMode(GL_PROJECTION);
	glPopMatrix(); // pop projection
}

void Visualization::setBallsLeft(int balls)
{
	m_balls_left = balls;
}