#pragma once

#ifdef WIN32
#include <windows.h>
#endif

#include <GL/gl.h>										
#include <GL/glu.h>
#include <GL/glut.h>
#include "SDL.h"
#include "tracker/math3d.h"
#include "tracker/ImageSource.h"
#include "tracker/MarkerTracker.h"
#include "gameobject.h"
#include "Board.h"
#include "Panel.h"
#include "Bumper.h"
#include "Ball.h"
#include "TextScreen.h"
#include "Box.h"

enum VIS_GAMEMODE { GAME_MENU = 0, GAME_PLAYING, GAME_PAUSED };
enum VIS_MARKERS { MARKER_BOARD = 0x0066, MARKER_BUMPER1 = 0x9dd0, MARKER_BUMPER2 = 0x6066, MARKER_BUMPER3 = 0x3BB8};

class Visualization
{
private:
	float m_projectionMatrix[16];
	int m_width;
	int m_height;
	int m_mode;			// 0 = GUI, 1 = GAME
	int m_time_passed;
	Physics* m_physics;
	float m_game_fading;
	float m_game_fade_speed;
	GameObject* m_GUI_background;
	void displayGUI();
	void displayGame();

	/// Function to display the game stats, like points, balls left, ...
	void displayHUD();
	void RenderString(float x, float y, float z,const char *string);

	ImageSource* m_source;

	/** @brief Directly load transform into OpenGL
	* @param t Transform loaded into GL
	*/
	void glLoadMatrixq(TransQuat t);

	/** @brief Directly multiply currently loaded Matrix with transform (OpenGL)
	* @param t Transform multiplied with current matrix in GL
	*/
	void glMultMatrixq(TransQuat t);

	/** @brief Get image from source and display it */
	void displayImage();

	void fadeGame();

	/// draw background-alpha-box for transperacy
	void drawAlphaBox();
	
	/// used to communicate the alpha-value
	float m_currentAlpha;

	/// display the main board
	Board * m_board;
	Panel * m_panel_left;  // We have two Panel Classes
	Panel * m_panel_right; // because they should store their own state

	Bumper * m_bumper1, * m_bumper2, * m_bumper3;	// Ok, we have one bumper model class

	TextScreen * m_textscreen;
	TextScreen * m_gameover;

	// left and right blocker so that the ball will not be lost that easily
	Box * m_rightBlocker, *m_leftBlocker;

	// Number of balls to go...
	int m_balls_left;
	
	float m_current_score;
	int m_current_bumperhits;

public:
	Ball *m_ball;	// here is our ball


	Visualization(int width, int height, MarkerTracker * tracker, ImageSource * source, Physics* physics);
	~Visualization(void);
	// for direct access to threshold (too lazy right now..)
	MarkerTracker* m_tracker;

	void init_gl();
	void display();
	/** Resize viewport when window dimensions changed
	* @param width New width
	* @param height New height
	*/
	void SizeOpenGLScreen(int width, int height);
	/// Switch to game mode
	void SetModeGame();
	/// Switch to GUI / Menu mode
	void SetModeGUI();

	/** Return current mode
	* @return current mode (enum: GAME_PLAYING or GAME_MENU atm)
	*/
	int GetCurrentMode();
	/** Create objects
	* @note This must be called after OpenGL is fully initialized, otherwise we lose.
	*/
	void init_objects();
	void fadeToBlack(void);
	void fadeToWhite(void);

	Panel * getLeftPanel();
	Panel * getRightPanel();

	/** Update the position of every object
	*/
	void updateObjects(void);

	void setBallsLeft(int balls);
};
