// AR-Praktikum ss2005 - Basti Grembowietz

#include "ImageSource.h"
#include <exception>

ImageSource::ImageSource()
{
	// initialize variables
	m_BGRImagePacked = NULL;
	m_image = NULL;
	m_width = m_height = m_numberOfChannels = m_depthPerChannel = 0;
	m_fullyInitialized = false;
}

ImageSource::~ImageSource()
{
	// free allocated image
	if (m_BGRImagePacked != NULL)
		delete [] m_BGRImagePacked;
}

unsigned char * ImageSource::getBGRImage()
{
	// only once work for a still-image
	if (newPictureAvalilable() || (!m_fullyInitialized))
	{
		if (!m_fullyInitialized)
		{
			// if the image is not fully initialized yet this means that
			// a buffer has to be allocated first.

//			if (m_depthPerChannel * m_numberOfChannels * m_width != m_image->widthStep)
			// NOTE: optimizations later.
			m_BGRImagePacked = new unsigned char [m_depthPerChannel * m_numberOfChannels * m_width * m_height]; 
			m_fullyInitialized = true;	
		}
		update();

		// line width (in bytes)
		int lineWidth = m_width * m_depthPerChannel * m_numberOfChannels;
		for (int y = 0; y < m_height; y++)
			memcpy((void*)&m_BGRImagePacked[y * lineWidth], (void*)&m_image->imageData[y * m_image->widthStep], lineWidth);

	}
	return m_BGRImagePacked;
}

void ImageSource::initDimensions()
{
	if (m_image)
	{
		m_width = m_image->width;
		m_height = m_image->height;

		switch (m_image->depth)
		{
			case IPL_DEPTH_8U: // unsigned 8-bit integers
			case IPL_DEPTH_8S: // signed 8-bit integers
				m_depthPerChannel = 1; break;
			case IPL_DEPTH_16S: // signed 16-bit integers
				m_depthPerChannel = 2; break;
			case IPL_DEPTH_32S: // signed 32-bit integers
			case IPL_DEPTH_32F: // single precision floating-point numbers
				m_depthPerChannel = 4; break;
			case IPL_DEPTH_64F: // double precision floating-point numbers
				m_depthPerChannel = 8; break;
			default:
				throw std::exception("Unknown image depth.");
		}
		m_numberOfChannels = m_image->nChannels;
	}
	else
	{
		throw std::exception("Could not initialize image dimensions.");
	}
}

IplImage* ImageSource::getImage()
{
	return m_image;
}


BCamera::BCamera()
{
	initCam(-1);
}

BCamera::BCamera(int numOfCam)
{
	initCam(numOfCam);
}

void BCamera::initCam(int numOfCam)
{
	m_Camera = NULL;
	m_Camera = cvCaptureFromCAM( numOfCam );
	if (!m_Camera)	
		throw std::exception("Camera could not be initialized properly.");
	
	// get first image to initialize the dimensions properly
	update();
	// init width, heigth and depth
	initDimensions();
}

BCamera::~BCamera()
{
	// properly release camera
	cvReleaseCapture(&m_Camera);
}

void BCamera::update()
{
	m_image = NULL;
	// aquire image
	m_image = cvQueryFrame(m_Camera);
	
	if (!m_image)
		throw std::exception("Could not query frame from camera.");
}

bool BCamera::newPictureAvalilable()
{
	return true;
}

BPicture::BPicture(std::string fileName)
{
	m_image = NULL;
	// load image from file
	m_image = cvLoadImage(fileName.c_str());

	if (!m_image)
	{
		std::string message = "Could load picture \"" + fileName + "\" .";
		throw std::exception(message.c_str());
	}
	
	initDimensions();
}

BPicture::~BPicture()
{
	cvReleaseImage(&m_image);
}

bool BPicture::newPictureAvalilable()
{
	return false;
}

void BPicture::update()
{
	// nothing to do here...
};