// AR-Praktikum ss2005 - Basti Grembowietz
// this classes are needed to use the source-code w/o having the need of a camera at all times...
#pragma once

#include <cv.h>
#include <highgui.h>

#include <string>

/** \brief Class abstracting the image sources
 * \ingroup AR Praktikum
 *
 *
 * \par requirements
 * win98 or later\n
 * win2k or later\n
 * OpenCV beta 4 or later\n
 * MFC\n
 *
 * \version 1.0
 * first version
 *
 * \date 05-29-2005
 *
 * \author Basti Grembowietz
 *
 * \par license
 * This code is mine. The code is provided "as is" with
 * no expressed or implied warranty. The author accepts no liability if it causes
 * any damage to your computer, causes your pet to fall ill, increases baldness
 * or makes your car start emitting strange noises when you start it up.
 * This code has no bugs, just undocumented features!
 * 
 * @todo Add support for movie files.
 */
class ImageSource
{
protected: 
	/** @brief The rgb-Image (packed) */
	unsigned char * m_BGRImagePacked;
	/** @brief Dimensions of the current image */
	int m_width, m_height, m_numberOfChannels, m_depthPerChannel;
	/** @brief Called to initialize the width, heigth and depth of an image*/
	void initDimensions();
	/** @brief check whether the image source is fully initialized
	*
	* This is used to allocate the RGB-Image buffer, it will only once be allocated.
	*/
	bool m_fullyInitialized;
	/** @brief My picture */
    IplImage * m_image;

public:
	/** @brief std. constructor */
	ImageSource();
	/** @brief std. destructor */
	virtual ~ImageSource();

	/** @brief Acquire image from source
	* @return The image
	* @note Do not release the pointer returned!
	*/
	IplImage* getImage();

	/** @brief Check if a new picture is avaliable
	*
	* This can be used in case of still-images that have no update, 
	* a camera or movie as source could just always return true.
	*/
	virtual bool newPictureAvalilable() = 0;

	/** @brief Convert BGR-image with "widthStep" into packed BGR
	*
	* In order to separate OpenCV from the Visualization, this function converts
	* the image aquired (by cam or still-image) into RGB in an allocated buffer
	* that will be returned to the caller.
	* @return Pointer to the converted image
	* @note Do not release the pointer returned!
	*/
	unsigned char * getBGRImage();

	/** @brief Getter for width
	* @return Width of the image
	*/
	int getWidth() {return m_width;};
	/** @brief Getter for height
	* @return Height of the image
	*/
	int getHeight() {return m_height;};
	/** @brief Getter for number of channels
	* @return Number of channels of the image
	*/
	int getNumberOfChannels() {return m_numberOfChannels;};
	/** @brief Getter for depth per channel
	* @return Depth per channel of the image
	*/
	int getDepthPerChannel() {return m_depthPerChannel;};

	/** @brief Aquire new picture (if possible)
	*/
	virtual void update() = 0;

};
/** @brief Handling the camera made easy 
 * \ingroup AR Praktikum
 *
 *
 * \par requirements
 * win98 or later\n
 * win2k or later\n
 * OpenCV beta 4 or later\n
 * MFC\n
 *
 * \version 1.0
 * first version
 *
 * \date 05-29-2005
 *
 * \author Basti Grembowietz
*/
class BCamera : public ImageSource
{
private:
	/** @brief My camera */
	CvCapture * m_Camera;

	/** @brief initialize camera 
	* @param[in] numOfCam number of the camera
	*/
	void initCam(int numOfCam);
public:
	/** @brief std. ctor */
	BCamera();
	/** @brief Ctor
	* @param[in] numOfCam Number of the camera to aquire
	*/
	BCamera(int numOfCam);
	/** @brief Dtor */
	~BCamera();
	
	bool newPictureAvalilable();

	void update();
};
/** @brief Handling a still image made easy 
 * \ingroup AR Praktikum
 *
 *
 * \par requirements
 * win98 or later\n
 * win2k or later\n
 * OpenCV beta 4 or later\n
 * MFC\n
 *
 * \version 1.0
 * first version
 *
 * \date 05-29-2005
 *
 * \author Basti Grembowietz
*/
class BPicture : public ImageSource
{
public:
	/** @brief std. ctor */
	BPicture(std::string fileName);
	/** @brief Dtor */
	~BPicture();
	
	bool newPictureAvalilable();

	void update();
};
