#pragma once
#include <cv.h>

/** 
 * computes the orientation and translation of a square
 * @param pRot result as quaternion
 * @param pTrans result position
 * @param p2D coordinates of the four corners in clock-wise order. 
 *        the origin is assumed to be at the camera's center of projection
 * @param fMarkerSize side-length of marker. Origin is at marker center.
 */
void estimateSquarePose( float* pRot, float *pTrans, const CvPoint2D32f* p2D, float fMarkerSize );

/** 
 * computes the orientation and translation of a square using homography
 * @param pRot result as quaternion
 * @param pTrans result position
 * @param p2D four input coordinates. the origin is assumed to be at the camera's center of projection
 * @param fMarkerSize side-length of marker. Origin is at marker center.
 * @param f focal length
 */
void poseFromHomography( float* pRot, float *pTrans, const CvPoint2D32f* p2D, float fMarkerSize, float f );

/**
 * optimize a pose with levenberg-marquardt
 * @param pRotation rotation as quaternion, both used as output and initial value
 * @param pTranslation 3-element translation, both used as output and initial value
 * @param nPoints number of correspondences
 * @param p2D pointer to camera coordinates
 * @param p3D pointer to object coordinates
 * @param f focal length
 */
void optimizePose( float* pRotation, float* pTranslation, int nPoints, const CvPoint2D32f* p2D, const CvPoint3D32f* p3D, float f );

/** Why not use this =) */
void projectPoint( CvPoint2D32f& p2D, CvPoint3D32f p3D, const float* pRotation, const float* pTranslation, float f );

